import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/widgets/user_info_form_container.dart';
import 'package:flutter_firebase_basic_tools/logic/user_info_form_functions.dart';
import 'package:flutter_firebase_basic_tools/models/abei_account.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:flutter_firebase_basic_tools/logic/translate.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_firebase_basic_tools/flutter_firebase_basic_tools.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';

class Account extends StatefulWidget {
  const Account({
    @required this.topbarBackgroundColor,
    @required this.userInfoFormContainerColor1,
    @required this.userInfoFormContainerColor2,
    @required this.userInfoFormContainerColor3,
    @required this.saveButtonColor,
    @required this.itemsTextColor,
    @required this.backgroundImagePath,
    @required this.apiKey,
  });

  final Color topbarBackgroundColor;
  final Color userInfoFormContainerColor1;
  final Color userInfoFormContainerColor2;
  final Color userInfoFormContainerColor3;
  final Color saveButtonColor;
  final Color itemsTextColor;
  final String backgroundImagePath;
  final String apiKey;

  @override
  _AccountState createState() => _AccountState();
}

class _AccountState extends State<Account> {
  final GlobalKey<ScaffoldState> scaffoldKey = new GlobalKey<ScaffoldState>();

  AbeiAccount args;

  String currentLanguage = "en";

  bool isEditingEmail = false;
  bool isEditingPassword = false;
  bool isEditingNameAndInfo = false;

  String birthdayLabel = "Enter your birthday";

  Timestamp birthdayTimestamp;

  CustomColors customColors = CustomColors();

  final GlobalKey<FormState> emailFormKey = GlobalKey<FormState>();
  final GlobalKey<FormState> passwordFormKey = GlobalKey<FormState>();
  final GlobalKey<FormState> nameAndInfoFormKey = GlobalKey<FormState>();

  TextEditingController firstNameInputController = TextEditingController();
  TextEditingController lastNameInputController = TextEditingController();
  TextEditingController emailInputController = TextEditingController();
  TextEditingController confirmEmailInputController = TextEditingController();
  TextEditingController passwordInputController = TextEditingController();
  TextEditingController confirmPasswordInputController =
      TextEditingController();

  String genderValue = 'Male';
  List<String> genderValues = [
    'Male',
    'Female',
    'Non-binary',
    'Rather not say',
  ];

  String countryValue = 'United States';
  List<String> countryValues = [
    'United States',
    'Mexico',
    'Canada',
    'Brazil',
  ];

  List<String> textList = [
    "Firstname",
    "Lastname",
    "Email",
    "Confirm email",
    "New password",
    "Confirm new password",
    "Enter your birthday",
    "Save",
  ];

  DateTime selectedDate = DateTime(DateTime.now().year - 18);

  String userID;

  bool filledFields = false;

  Future<Null> _selectDate(BuildContext context) async {
    final DateTime picked = await showDatePicker(
      context: context,
      initialDate: selectedDate,
      firstDate: DateTime(DateTime.now().year - 100),
      lastDate: DateTime(DateTime.now().year - 18),
    );
    if (picked != null && picked != selectedDate)
      setState(() {
        selectedDate = picked;

        DateFormat dateFormatter = DateFormat.yMMMMd('en_US');
        String dateNowFormatted = dateFormatter.format(selectedDate);
        birthdayLabel = dateNowFormatted;
      });
  }

  SharedPreferences prefs;
  TranslationStream translationStream = TranslationStream();

  int flexIndexForEditingContainer = 10;

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
  }

  showAlertDialog(BuildContext context) {
    showDialog(
        context: context,
        builder: (BuildContext context) {
          return AlertDialog(
            title: Text("Do you want to save the changes?"),
            actions: <Widget>[
              FlatButton(
                child: Text("Discard"),
                onPressed: () {
                  Navigator.of(context).pop();

                  if (isEditingEmail) {
                    isEditingEmail = false;
                  } else if (isEditingPassword) {
                    isEditingPassword = false;
                  } else if (isEditingNameAndInfo) {
                    isEditingNameAndInfo = false;
                  }
                },
              ),
              FlatButton(
                child: Text("Cancel"),
                onPressed: () {
                  Navigator.of(context).pop();
                },
              ),
              FlatButton(
                child: Text("Accept"),
                onPressed: () {
                  if (isEditingEmail) {
                    List<TextEditingController> inputControllers = [
                      emailInputController,
                      confirmEmailInputController,
                    ];

                    UserInfoFormFunctions().updateUserEmail(
                      context: context,
                      scaffoldKey: scaffoldKey,
                      emailFormKey: emailFormKey,
                      inputControllers: inputControllers,
                      userID: userID,
                      isEditingEmail: isEditingEmail,
                      widgetParent: this,
                    );
                  } else if (isEditingPassword) {
                    List<TextEditingController> inputControllers = [
                      passwordInputController,
                      confirmPasswordInputController,
                    ];

                    UserInfoFormFunctions().updateUserPassword(
                      context: context,
                      scaffoldKey: scaffoldKey,
                      passwordFormKey: passwordFormKey,
                      inputControllers: inputControllers,
                      userID: userID,
                      isEditingPassword: isEditingPassword,
                      widgetParent: this,
                    );
                  } else if (isEditingNameAndInfo) {
                    List<TextEditingController> inputControllers = [
                      firstNameInputController,
                      lastNameInputController,
                    ];

                    UserInfoFormFunctions().updateUserNameAndInfo(
                      context: context,
                      scaffoldKey: scaffoldKey,
                      nameAndInfoFormKey: nameAndInfoFormKey,
                      inputControllers: inputControllers,
                      selectedDate: selectedDate,
                      genderValue: genderValue,
                      countryValue: countryValue,
                      userID: userID,
                      isEditingNameAndInfo: isEditingNameAndInfo,
                      widgetParent: this,
                    );
                  }
                },
              )
            ],
          );
        });
  }

  fillFields() async {
    if (!filledFields) {
      filledFields = true;

      DocumentSnapshot firestoreUser =
          await FirebaseFirestore.instance.collection("users").doc(userID).get();

      print(firestoreUser.data);

      User user = FirebaseAuth.instance.currentUser;

      firstNameInputController.text = firestoreUser.data()["firstname"];
      lastNameInputController.text = firestoreUser.data()["lastname"];
      emailInputController.text = user.email;
      confirmEmailInputController.text = user.email;
      passwordInputController.text = "00000000";
      confirmPasswordInputController.text = "00000000";

      genderValue = firestoreUser.data()["gender"];
      countryValue = firestoreUser.data()["country"];

      setState(() {});
    }
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
    translationStream.init(textList, this);
    translationStream.translate(widget.apiKey);
  }

  @override
  Widget build(BuildContext context) {
    args = ModalRoute.of(context).settings.arguments;

    userID = args.userID;

    fillFields();

    bool isLandscape =
        (MediaQuery.of(context).orientation == Orientation.landscape);

    double itemGroupHeight = 0;
    double itemGroupTopMargin = 0;

    if (isLandscape) {
      itemGroupHeight = MediaQuery.of(context).size.height / 12;
      itemGroupTopMargin = MediaQuery.of(context).size.height / 80;
    } else {
      itemGroupHeight = MediaQuery.of(context).size.height / 6;
      itemGroupTopMargin = MediaQuery.of(context).size.height / 40;
    }

    isLandscape
        ? flexIndexForEditingContainer = 10
        : flexIndexForEditingContainer = 4;

    double appBarSize = 65;

    return Scaffold(
      key: scaffoldKey,
      extendBodyBehindAppBar: false,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: customColors.green.withOpacity(0.7),
          size: appBarSize,
          hasBackButton: true,
          actions: <Widget>[
            Container(
              margin: EdgeInsets.only(right: 20),
              width: isLandscape ? 200 : 100,
              child: LanguagePicker(
                currentLanguage: currentLanguage,
                parentWidget: this,
                itemsTextColor: widget.itemsTextColor,
                apiKey: "[API_KEY]",
              ),
            ),
          ],
          customLeading: null,
        ),
      ),
      body: UserInfoFormContainer(
        userInfoFormType: UserInfoFormType.accountInfo,
        cardColor: Colors.white,
        backgroundImagePath: widget.backgroundImagePath,
        linearGradient: LinearGradient(
          begin: FractionalOffset.bottomRight,
          end: FractionalOffset.topLeft,
          colors: [
            widget.userInfoFormContainerColor1,
            widget.userInfoFormContainerColor2,
            widget.userInfoFormContainerColor3,
          ],
          stops: [
            0.0,
            0.4,
            1.0,
          ],
        ),
        child: LayoutBuilder(builder:
            (BuildContext context, BoxConstraints viewportConstraints) {
          return SingleChildScrollView(
            child: ConstrainedBox(
              constraints: BoxConstraints(
                minHeight: viewportConstraints.minHeight,
              ),
              child: IntrinsicHeight(
                child: Column(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: <Widget>[
                    Form(
                      key: emailFormKey,
                      child: Row(
                        children: <Widget>[
                          Expanded(
                            flex: flexIndexForEditingContainer,
                            child: Container(
                              height: itemGroupHeight,
                              margin: EdgeInsets.only(
                                top: itemGroupTopMargin,
                              ),
                              child: Flex(
                                mainAxisSize: MainAxisSize.min,
                                direction: isLandscape
                                    ? Axis.horizontal
                                    : Axis.vertical,
                                children: <Widget>[
                                  Expanded(
                                    flex: 10,
                                    child: TextFormField(
                                      //enabled: isEditingEmail,
                                      decoration: InputDecoration(
                                        labelText: textList[2],
                                      ),
                                      controller: emailInputController,
                                      validator: (String value) =>
                                          FormFieldValidators(
                                        value: value,
                                        type: FormFieldValidatorsType.email,
                                      ).validate(),
                                    ),
                                  ),
                                  Spacer(flex: 1),
                                  Expanded(
                                    flex: 10,
                                    child: TextFormField(
                                      //enabled: isEditingEmail,
                                      decoration: InputDecoration(
                                        labelText: textList[3],
                                      ),
                                      controller: confirmEmailInputController,
                                      validator: (String value) =>
                                          FormFieldValidators(
                                        value: value,
                                        type: FormFieldValidatorsType.email,
                                      ).validate(),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ),
                          Expanded(
                            flex: 1,
                            child: Container(
                              height: itemGroupHeight,
                              child: Column(
                                children: <Widget>[
                                  IconButton(
                                    padding: EdgeInsets.all(0),
                                    icon: new Icon(
                                      Icons.edit,
                                      color: customColors.lightAqua,
                                    ),
                                    onPressed: () {
                                      if (!isEditingEmail &&
                                          !isEditingPassword &&
                                          !isEditingNameAndInfo) {
                                        isEditingEmail = true;

                                        setState(() {});
                                      }
                                    },
                                  ),
                                  Spacer(flex: 1),
                                ],
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                    Form(
                      key: passwordFormKey,
                      child: Row(
                        children: <Widget>[
                          Expanded(
                            flex: flexIndexForEditingContainer,
                            child: Container(
                              height: itemGroupHeight,
                              margin: EdgeInsets.only(
                                top: itemGroupTopMargin,
                              ),
                              child: Flex(
                                direction: isLandscape
                                    ? Axis.horizontal
                                    : Axis.vertical,
                                children: <Widget>[
                                  Expanded(
                                    flex: 10,
                                    child: TextFormField(
                                      enabled: isEditingPassword,
                                      decoration: InputDecoration(
                                        labelText: textList[4],
                                      ),
                                      controller: passwordInputController,
                                      validator: (String value) =>
                                          FormFieldValidators(
                                        value: value,
                                        type: FormFieldValidatorsType.password,
                                      ).validate(),
                                      obscureText: true,
                                    ),
                                  ),
                                  Spacer(flex: 1),
                                  Expanded(
                                    flex: 10,
                                    child: TextFormField(
                                      enabled: isEditingPassword,
                                      decoration: InputDecoration(
                                        labelText: textList[5],
                                      ),
                                      controller:
                                          confirmPasswordInputController,
                                      validator: (String value) =>
                                          FormFieldValidators(
                                        value: value,
                                        type: FormFieldValidatorsType.password,
                                      ).validate(),
                                      obscureText: true,
                                    ),
                                  ),
                                ],
                              ),
                            ),
                          ),
                          Expanded(
                            flex: 1,
                            child: Container(
                              height: itemGroupHeight,
                              child: Column(
                                children: <Widget>[
                                  IconButton(
                                    padding: EdgeInsets.all(0),
                                    icon: new Icon(
                                      Icons.edit,
                                      color: customColors.lightAqua,
                                    ),
                                    onPressed: () {
                                      if (!isEditingEmail &&
                                          !isEditingPassword &&
                                          !isEditingNameAndInfo) {
                                        isEditingPassword = true;

                                        passwordInputController.text = "";
                                        confirmPasswordInputController.text =
                                            "";

                                        setState(() {});
                                      }
                                    },
                                  ),
                                  Spacer(flex: 1),
                                ],
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                    Form(
                      key: nameAndInfoFormKey,
                      child: Row(
                        children: <Widget>[
                          Expanded(
                            flex: flexIndexForEditingContainer,
                            child: Column(
                              children: <Widget>[
                                Container(
                                  height: itemGroupHeight,
                                  margin: EdgeInsets.only(
                                    top: itemGroupTopMargin,
                                  ),
                                  child: Flex(
                                    direction: isLandscape
                                        ? Axis.horizontal
                                        : Axis.vertical,
                                    children: <Widget>[
                                      Expanded(
                                        flex: 10,
                                        child: TextFormField(
                                          enabled: isEditingNameAndInfo,
                                          decoration: InputDecoration(
                                            labelText: textList[0],
                                          ),
                                          controller: firstNameInputController,
                                          validator: (String value) =>
                                              FormFieldValidators(
                                            value: value,
                                            type: FormFieldValidatorsType
                                                .firstname,
                                          ).validate(),
                                        ),
                                      ),
                                      Spacer(flex: 1),
                                      Expanded(
                                        flex: 10,
                                        child: TextFormField(
                                          enabled: isEditingNameAndInfo,
                                          decoration: InputDecoration(
                                            labelText: textList[1],
                                          ),
                                          controller: lastNameInputController,
                                          validator: (String value) =>
                                              FormFieldValidators(
                                                      value: value,
                                                      type:
                                                          FormFieldValidatorsType
                                                              .lastname)
                                                  .validate(),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                                Container(
                                  height: itemGroupHeight / 2,
                                  width: MediaQuery.of(context).size.width,
                                  margin: EdgeInsets.only(
                                    top: itemGroupTopMargin,
                                  ),
                                  padding: EdgeInsets.all(0),
                                  child: FractionallySizedBox(
                                    heightFactor: isLandscape ? 1 : 0.5,
                                    child: RaisedButton(
                                      onPressed: isEditingNameAndInfo
                                          ? () {
                                              _selectDate(context);
                                            }
                                          : null,
                                      elevation: 0,
                                      color: Colors.transparent,
                                      shape: RoundedRectangleBorder(
                                        borderRadius: BorderRadius.circular(5),
                                        side: BorderSide(
                                          color: customColors.lightAqua,
                                        ),
                                      ),
                                      child: Text(
                                        birthdayLabel != "Enter your birthday"
                                            ? birthdayLabel
                                            : textList[6],
                                        style: TextStyle(
                                          color: isEditingNameAndInfo
                                              ? customColors.lightAqua
                                              : Colors.white,
                                        ),
                                      ),
                                    ),
                                  ),
                                ),
                                Container(
                                  height: itemGroupHeight,
                                  margin: EdgeInsets.only(
                                    top: itemGroupTopMargin,
                                  ),
                                  child: Flex(
                                    direction: isLandscape
                                        ? Axis.horizontal
                                        : Axis.vertical,
                                    //direction: Axis.horizontal,
                                    mainAxisAlignment:
                                        MainAxisAlignment.spaceEvenly,
                                    children: <Widget>[
                                      Expanded(
                                        flex: 10,
                                        child: DropdownButton<String>(
                                          isExpanded: true,
                                          value: genderValue,
                                          iconSize: 24,
                                          elevation: 16,
                                          style: TextStyle(
                                              color: CustomColors().oceanBlue),
                                          underline: Container(
                                            height: 2,
                                            color: CustomColors().oceanBlue,
                                          ),
                                          onChanged: isEditingNameAndInfo
                                              ? (String newValue) {
                                                  setState(() {
                                                    genderValue = newValue;
                                                  });
                                                }
                                              : null,
                                          items: genderValues
                                              .map<DropdownMenuItem<String>>(
                                                  (String value) {
                                            return DropdownMenuItem<String>(
                                              value: value,
                                              child: Text(value),
                                            );
                                          }).toList(),
                                        ),
                                      ),
                                      Spacer(flex: 1),
                                      Expanded(
                                        flex: 10,
                                        child: DropdownButton<String>(
                                          isExpanded: true,
                                          value: countryValue,
                                          iconSize: 24,
                                          elevation: 16,
                                          style: TextStyle(
                                              color: CustomColors().oceanBlue),
                                          underline: Container(
                                            height: 2,
                                            color: CustomColors().oceanBlue,
                                          ),
                                          onChanged: isEditingNameAndInfo
                                              ? (String newValue) {
                                                  setState(() {
                                                    countryValue = newValue;
                                                  });
                                                }
                                              : null,
                                          items: countryValues
                                              .map<DropdownMenuItem<String>>(
                                                  (String value) {
                                            return DropdownMenuItem<String>(
                                              value: value,
                                              child: Text(value),
                                            );
                                          }).toList(),
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                              ],
                            ),
                          ),
                          Expanded(
                            flex: 1,
                            child: Container(
                              height:
                                  (itemGroupHeight * 2) + (itemGroupHeight / 2),
                              child: Column(
                                children: <Widget>[
                                  IconButton(
                                    padding: EdgeInsets.all(0),
                                    icon: new Icon(
                                      Icons.edit,
                                      color: customColors.lightAqua,
                                    ),
                                    onPressed: () {
                                      if (!isEditingEmail &&
                                          !isEditingPassword &&
                                          !isEditingNameAndInfo) {
                                        isEditingNameAndInfo = true;
                                        setState(() {});
                                      }
                                    },
                                  ),
                                  Spacer(flex: 1),
                                ],
                              ),
                            ),
                          ),
                        ],
                      ),
                    ),
                    Row(
                      children: <Widget>[
                        Expanded(
                          flex: flexIndexForEditingContainer,
                          child: Container(
                            width: MediaQuery.of(context).size.width,
                            padding: EdgeInsets.all(0),
                            margin: EdgeInsets.only(
                              top: itemGroupTopMargin,
                              bottom: itemGroupTopMargin,
                            ),
                            child: RaisedButton(
                              color: widget.saveButtonColor,
                              onPressed: isEditingEmail ||
                                      isEditingPassword ||
                                      isEditingNameAndInfo
                                  ? () {
                                      showAlertDialog(context);
                                    }
                                  : null,
                              shape: RoundedRectangleBorder(
                                borderRadius: BorderRadius.circular(5),
                              ),
                              child: Text(
                                textList[7],
                                textAlign: TextAlign.center,
                                style: TextStyle(
                                  color: Colors.white,
                                  fontSize: 12,
                                  //fontWeight: FontWeight.bold,
                                ),
                              ),
                            ),
                          ),
                        ),
                        Spacer(flex: 1),
                      ],
                    ),
                  ],
                ),
              ),
            ),
          );
        }),
      ),
    );
  }
}
