import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/widgets/user_info_form_container.dart';
import 'package:flutter_firebase_basic_tools/logic/user_info_form_functions.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:flutter_firebase_basic_tools/logic/translate.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_firebase_basic_tools/flutter_firebase_basic_tools.dart';
import 'package:intl/intl.dart';
import 'package:shared_preferences/shared_preferences.dart';

class Register extends StatefulWidget {
  const Register({
    @required this.topbarBackgroundColor,
    @required this.userInfoFormContainerColor1,
    @required this.userInfoFormContainerColor2,
    @required this.userInfoFormContainerColor3,
    @required this.registerButtonColor,
    @required this.itemsTextColor,
    @required this.backgroundImagePath,
    @required this.apiKey,
  });

  final Color topbarBackgroundColor;
  final Color userInfoFormContainerColor1;
  final Color userInfoFormContainerColor2;
  final Color userInfoFormContainerColor3;
  final Color registerButtonColor;
  final Color itemsTextColor;
  final String backgroundImagePath;
  final String apiKey;

  @override
  _RegisterState createState() => _RegisterState();
}

class _RegisterState extends State<Register> {
  final GlobalKey<ScaffoldState> scaffoldKey = new GlobalKey<ScaffoldState>();

  String currentLanguage = "en";

  bool acceptTerms = false;

  String birthdayLabel = "Enter your birthday";

  Timestamp birthdayTimestamp;

  CustomColors customColors = CustomColors();

  final GlobalKey<FormState> registerFormKey = GlobalKey<FormState>();

  TextEditingController firstNameInputController = TextEditingController();
  TextEditingController lastNameInputController = TextEditingController();
  TextEditingController emailInputController = TextEditingController();
  TextEditingController confirmEmailInputController = TextEditingController();
  TextEditingController passwordInputController = TextEditingController();
  TextEditingController confirmPasswordInputController =
      TextEditingController();

  String genderValue = 'Male';
  List<String> genderValues = [
    'Male',
    'Female',
    'Non-binary',
    'Rather not say',
  ];

  String countryValue = 'United States';
  List<String> countryValues = [
    'United States',
    'Mexico',
    'Canada',
    'Brazil',
  ];

  List<String> textList = [
    "Firstname",
    "Lastname",
    "Email",
    "Confirm email",
    "Password",
    "Confirm password",
    "Enter your birthday",
    "I accept the Terms of Use and Privacy Policy",
    "Register",
  ];

  DateTime selectedDate = DateTime(DateTime.now().year - 18);

  Future<Null> _selectDate(BuildContext context) async {
    final DateTime picked = await showDatePicker(
      context: context,
      initialDate: selectedDate,
      firstDate: DateTime(DateTime.now().year - 100),
      lastDate: DateTime(DateTime.now().year - 18),
    );
    if (picked != null && picked != selectedDate)
      setState(() {
        selectedDate = picked;

        DateFormat dateFormatter = DateFormat.yMMMMd('en_US');
        String dateNowFormatted = dateFormatter.format(selectedDate);
        birthdayLabel = dateNowFormatted;
      });
  }

  SharedPreferences prefs;
  TranslationStream translationStream = TranslationStream();

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
    translationStream.init(textList, this);
    translationStream.translate(widget.apiKey);
  }

  @override
  Widget build(BuildContext context) {
    bool isLandscape =
        (MediaQuery.of(context).orientation == Orientation.landscape);

    double itemGroupHeight = 0;
    double itemGroupTopMargin = 0;

    if (isLandscape) {
      itemGroupHeight = MediaQuery.of(context).size.height / 12;
      itemGroupTopMargin = MediaQuery.of(context).size.height / 80;
    } else {
      itemGroupHeight = MediaQuery.of(context).size.height / 6;
      itemGroupTopMargin = MediaQuery.of(context).size.height / 40;
    }

    double appBarSize = 65;

    return Scaffold(
      key: scaffoldKey,
      extendBodyBehindAppBar: false,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: widget.topbarBackgroundColor,
          size: appBarSize,
          hasBackButton: true,
          actions: <Widget>[
            Container(
              margin: EdgeInsets.only(right: 20),
              width: isLandscape ? 200 : 100,
              child: LanguagePicker(
                currentLanguage: currentLanguage,
                parentWidget: this,
                itemsTextColor: widget.itemsTextColor,
                apiKey: "[API_KEY]",
              ),
            ),
          ],
          customLeading: null,
        ),
      ),
      body: UserInfoFormContainer(
        userInfoFormType: UserInfoFormType.register,
        cardColor: Colors.white,
        backgroundImagePath: widget.backgroundImagePath,
        linearGradient: LinearGradient(
          begin: FractionalOffset.bottomRight,
          end: FractionalOffset.topLeft,
          colors: [
            widget.userInfoFormContainerColor1,
            widget.userInfoFormContainerColor2,
            widget.userInfoFormContainerColor3,
          ],
          stops: [
            0.0,
            0.4,
            1.0,
          ],
        ),
        child: Form(
          key: registerFormKey,
          child: LayoutBuilder(builder:
              (BuildContext context, BoxConstraints viewportConstraints) {
            return SingleChildScrollView(
              child: ConstrainedBox(
                constraints: BoxConstraints(
                  minHeight: viewportConstraints.minHeight,
                ),
                child: IntrinsicHeight(
                  child: Column(
                    children: <Widget>[
                      Container(
                        height: itemGroupHeight,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        child: Flex(
                          direction:
                              isLandscape ? Axis.horizontal : Axis.vertical,
                          children: <Widget>[
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[0],
                                ),
                                controller: firstNameInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                  value: value,
                                  type: FormFieldValidatorsType.firstname,
                                ).validate(),
                              ),
                            ),
                            Spacer(flex: 1),
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[1],
                                ),
                                controller: lastNameInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                            value: value,
                                            type: FormFieldValidatorsType
                                                .lastname)
                                        .validate(),
                              ),
                            ),
                          ],
                        ),
                      ),
                      Container(
                        height: itemGroupHeight,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        child: Flex(
                          direction:
                              isLandscape ? Axis.horizontal : Axis.vertical,
                          children: <Widget>[
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[2],
                                ),
                                controller: emailInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                  value: value,
                                  type: FormFieldValidatorsType.email,
                                ).validate(),
                              ),
                            ),
                            Spacer(flex: 1),
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[3],
                                ),
                                controller: confirmEmailInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                  value: value,
                                  type: FormFieldValidatorsType.email,
                                ).validate(),
                              ),
                            ),
                          ],
                        ),
                      ),
                      Container(
                        height: itemGroupHeight,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        child: Flex(
                          direction:
                              isLandscape ? Axis.horizontal : Axis.vertical,
                          children: <Widget>[
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[4],
                                ),
                                controller: passwordInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                  value: value,
                                  type: FormFieldValidatorsType.password,
                                ).validate(),
                                obscureText: true,
                              ),
                            ),
                            Spacer(flex: 1),
                            Expanded(
                              flex: 10,
                              child: TextFormField(
                                decoration: InputDecoration(
                                  labelText: textList[5],
                                ),
                                controller: confirmPasswordInputController,
                                validator: (String value) =>
                                    FormFieldValidators(
                                  value: value,
                                  type: FormFieldValidatorsType.password,
                                ).validate(),
                                obscureText: true,
                              ),
                            ),
                          ],
                        ),
                      ),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        padding: EdgeInsets.all(0),
                        child: RaisedButton(
                          onPressed: () {
                            _selectDate(context);
                          },
                          elevation: 0,
                          color: Colors.transparent,
                          shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(5),
                            side: BorderSide(
                              color: customColors.lightAqua,
                            ),
                          ),
                          child: Text(
                            birthdayLabel != "Enter your birthday"
                                ? birthdayLabel
                                : textList[6],
                            style: TextStyle(
                              color: customColors.lightAqua,
                            ),
                          ),
                        ),
                      ),
                      Container(
                        height: itemGroupHeight,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        child: Flex(
                          direction:
                              isLandscape ? Axis.horizontal : Axis.vertical,
                          //direction: Axis.horizontal,
                          mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                          children: <Widget>[
                            Expanded(
                              flex: 10,
                              child: DropdownButton<String>(
                                isExpanded: true,
                                value: genderValue,
                                iconSize: 24,
                                elevation: 16,
                                style:
                                    TextStyle(color: CustomColors().oceanBlue),
                                underline: Container(
                                  height: 2,
                                  color: CustomColors().oceanBlue,
                                ),
                                onChanged: (String newValue) {
                                  setState(() {
                                    genderValue = newValue;
                                  });
                                },
                                items: genderValues
                                    .map<DropdownMenuItem<String>>(
                                        (String value) {
                                  return DropdownMenuItem<String>(
                                    value: value,
                                    child: Text(value),
                                  );
                                }).toList(),
                              ),
                            ),
                            Spacer(flex: 1),
                            Expanded(
                              flex: 10,
                              child: DropdownButton<String>(
                                isExpanded: true,
                                value: countryValue,
                                iconSize: 24,
                                elevation: 16,
                                style:
                                    TextStyle(color: CustomColors().oceanBlue),
                                underline: Container(
                                  height: 2,
                                  color: CustomColors().oceanBlue,
                                ),
                                onChanged: (String newValue) {
                                  setState(() {
                                    countryValue = newValue;
                                  });
                                },
                                items: countryValues
                                    .map<DropdownMenuItem<String>>(
                                        (String value) {
                                  return DropdownMenuItem<String>(
                                    value: value,
                                    child: Text(value),
                                  );
                                }).toList(),
                              ),
                            ),
                          ],
                        ),
                      ),
                      Container(
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                        ),
                        child: FlatButton(
                          padding: EdgeInsets.all(0),
                          onPressed: () {
                            acceptTerms = !acceptTerms;
                            setState(() {});
                          },
                          child: Row(
                            children: <Widget>[
                              Expanded(
                                flex: 1,
                                child: Container(
                                  child: Icon(
                                    acceptTerms
                                        ? Icons.check_box
                                        : Icons.check_box_outline_blank,
                                    color: CustomColors().oceanBlue,
                                  ),
                                  margin: EdgeInsets.only(
                                    right: 10,
                                  ),
                                ),
                              ),
                              Expanded(
                                flex: 7,
                                child: Text(
                                  textList[7],
                                  overflow: TextOverflow.clip,
                                  maxLines: 2,
                                ),
                              ),
                            ],
                          ),
                        ),
                      ),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                          top: itemGroupTopMargin,
                          bottom: itemGroupTopMargin,
                        ),
                        child: RaisedButton(
                          color: CustomColors().oceanBlue,
                          onPressed: () {
                            List<TextEditingController> inputControllers = [
                              firstNameInputController,
                              lastNameInputController,
                              emailInputController,
                              confirmEmailInputController,
                              passwordInputController,
                              confirmPasswordInputController,
                            ];

                            UserInfoFormFunctions().register(
                              context: context,
                              scaffoldKey: scaffoldKey,
                              acceptTerms: acceptTerms,
                              registerFormKey: registerFormKey,
                              inputControllers: inputControllers,
                              selectedDate: selectedDate,
                              genderValue: genderValue,
                              countryValue: countryValue,
                            );
                          },
                          child: Text(
                            textList[8],
                            textAlign: TextAlign.center,
                            style: TextStyle(
                              color: Colors.white,
                              fontSize: 12,
                              //fontWeight: FontWeight.bold,
                            ),
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            );
          }),
        ),
      ),
    );
  }
}
