import 'dart:ui';

import 'package:flutter/material.dart';

class BackgroundImageWithGradientColor extends StatelessWidget {
  const BackgroundImageWithGradientColor({
    @required this.child,
    @required this.backgroundImagePath,
    @required this.linearGradient,
    @required this.boxFit,
    @required this.height,
    @required this.width,
    this.blurImage,
    this.blurImageColor,
    this.blurImageParameters,
  });

  final Widget child;
  final String backgroundImagePath;
  final LinearGradient linearGradient;
  final BoxFit boxFit;
  final bool blurImage;
  final Color blurImageColor;
  final List<double> blurImageParameters;
  final double height;
  final double width;

  @override
  Widget build(BuildContext context) {
    return Container(
      height: height,
      width: width,
      child: Stack(
        alignment: Alignment.center,
        children: <Widget>[
          Container(
            height: height,
            width: width,
            decoration: BoxDecoration(
              image: backgroundImagePath != null
                  ? DecorationImage(
                      fit: boxFit,
                      image: backgroundImagePath.contains("http") ||
                              backgroundImagePath.contains(".com")
                          ? Image.network(
                              backgroundImagePath,
                            ).image
                          : AssetImage(
                              backgroundImagePath,
                            ),
                    )
                  : null,
            ),
            child: blurImage != null && blurImage
                ? BackdropFilter(
                    filter: ImageFilter.blur(
                      sigmaX: blurImageParameters[0],
                      sigmaY: blurImageParameters[1],
                    ),
                    child: Container(
                      color: blurImageColor,
                    ),
                  )
                : Container(),
          ),
          Container(
            height: height,
            width: width,
            decoration: BoxDecoration(gradient: linearGradient),
          ),
          child,
        ],
      ),
    );
  }
}
