import 'package:flutter_firebase_basic_tools/widgets/certificate_visualizer.dart';
import 'package:flutter_firebase_basic_tools/widgets/covered_container_coming_soon.dart';
import 'package:flutter_firebase_basic_tools/logic/get_user_info.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:flutter_firebase_basic_tools/logic/timestamp_to_date.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class CertificatesAndRewards extends StatefulWidget {
  @override
  _CertificatesAndRewardsState createState() => _CertificatesAndRewardsState();
}

class _CertificatesAndRewardsState extends State<CertificatesAndRewards> {
  double currentPage = 0;

  CustomColors customColors = CustomColors();

  final PageController pageController = PageController(initialPage: 0);

  SharedPreferences prefs;

  List certificatesID = List();
  List<Map<String, dynamic>> certificates = List<Map<String, dynamic>>();

  Map<String, dynamic> userInfo = {};

  String uid;

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
    uid = prefs.getString("uid");
    getCertificates();
  }

  getCertificates() async {
    userInfo = await getUserInfo(uid);
    certificates.clear();

    print(userInfo);

    if (userInfo["certificates"] != null) {
      if (userInfo["certificates"].length > 0) {
        certificatesID = List.from(userInfo["certificates"]);

        print(certificatesID);

        for (int i = 0; i < certificatesID.length; i++) {
          DocumentSnapshot firestoreCertificate = await FirebaseFirestore.instance
              .collection("certificates")
              .doc(certificatesID[i])
              .get();

          DocumentSnapshot firestoreCourse = await FirebaseFirestore.instance
              .collection("courses")
              .doc(firestoreCertificate.data()["course_id"])
              .get();

          DocumentSnapshot firestoreUser = await FirebaseFirestore.instance
              .collection("users")
              .doc(firestoreCertificate.data()["user_id"])
              .get();

          print(firestoreCertificate.data);
          print(firestoreCourse.data);
          print(firestoreUser.data);

          certificates.add({
            "date": firestoreCertificate.data()["date"],
            "course_name": firestoreCourse.data()["name"],
            "user_name": firestoreUser.data()["firstname"] +
                " " +
                firestoreUser.data()["lastname"],
            "id": certificatesID[i],
          });
        }
      }
    }

    print("certificates: " + certificates.toString());

    setState(() {});
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Courses'),
        backgroundColor: customColors.darkAqua,
        actions: <Widget>[
          Container(
            height: MediaQuery.of(context).size.height / 15,
            width: MediaQuery.of(context).size.height / 15,
            margin: EdgeInsets.only(
              right: MediaQuery.of(context).size.width / 10,
            ),
            decoration: BoxDecoration(
              image: DecorationImage(
                fit: BoxFit.fitHeight,
                image: AssetImage(
                  "assets/images/abei_logo.png",
                ),
              ),
            ),
          ),
        ],
      ),
      body: Column(
        children: [
          Spacer(flex: 1),
          Expanded(
            flex: 1,
            child: Container(
              child: Row(
                children: <Widget>[
                  Spacer(flex: 3),
                  Expanded(
                    flex: 1,
                    child: RaisedButton(
                      onPressed: () {},
                      elevation: 0,
                      color: currentPage == 0
                          ? customColors.oceanBlue
                          : Colors.white,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(5),
                        side: BorderSide(
                          color: customColors.oceanBlue,
                        ),
                      ),
                      child: Text(
                        'Certificates',
                        style: TextStyle(
                          color: currentPage == 0
                              ? Colors.white
                              : customColors.lightAqua,
                        ),
                      ),
                    ),
                  ),
                  Spacer(flex: 1),
                  Expanded(
                    flex: 1,
                    child: RaisedButton(
                      onPressed: () {},
                      elevation: 0,
                      color: currentPage == 1
                          ? customColors.oceanBlue
                          : Colors.white,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(5),
                        side: BorderSide(
                          color: customColors.oceanBlue,
                        ),
                      ),
                      child: Text(
                        'Rewards',
                        style: TextStyle(
                          color: currentPage == 1
                              ? Colors.white
                              : customColors.lightAqua,
                        ),
                      ),
                    ),
                  ),
                  Spacer(flex: 3),
                ],
              ),
            ),
          ),
          Spacer(flex: 1),
          Expanded(
            flex: 7,
            child: Stack(
              children: <Widget>[
                PageView(
                  onPageChanged: (int page) {
                    setState(() {
                      currentPage = page.toDouble();
                    });
                  },
                  controller: pageController,
                  children: [
                    Container(
                      child: certificatesID.length > 0
                          ? FractionallySizedBox(
                              widthFactor: 0.5,
                              child: Container(
                                height:
                                    MediaQuery.of(context).size.height / 1.5,
                                child: ListView.builder(
                                  itemCount: certificates.length,
                                  itemBuilder: (context, i) {
                                    return GestureDetector(
                                      onTap: () {
                                        print(i);
                                        Navigator.push(
                                          context,
                                          MaterialPageRoute(
                                              builder: (context) =>
                                                  CertificatesVisualizer(
                                                    id: certificates[i]["id"],
                                                    uid: uid,
                                                    userName: certificates[i]
                                                        ["user_name"],
                                                    courseName: certificates[i]
                                                        ["course_name"],
                                                    date: TimestampToDate()
                                                        .convert(certificates[i]
                                                            ["date"]),
                                                  )),
                                        );
                                      },
                                      child: ListTile(
                                        title: Text(
                                          "Date: ${TimestampToDate().convert(certificates[i]["date"])} \nCourse Name: ${certificates[i]["course_name"]} \nCertificate ID: ${certificates[i]["id"]}",
                                          style: TextStyle(fontSize: 18.0),
                                        ),
                                        leading: Icon(
                                          Icons.account_circle,
                                          color: Colors.lightGreen,
                                        ),
                                      ),
                                    );
                                  },
                                ),
                              ),
                            )
                          : Center(
                              child: CircularProgressIndicator(
                                valueColor: new AlwaysStoppedAnimation<Color>(
                                  CustomColors().lightAqua,
                                ),
                              ),
                            ),
                    ),
                    CoveredContainerComingSoon(
                      enableCover: true,
                      child: Container(
                        child: Text(
                          "",
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            color: Colors.black,
                            fontSize: 40,
                            fontWeight: FontWeight.bold,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ],
            ),
          ),
          Spacer(flex: 1),
        ],
      ),
    );
  }
}

class Certificate {
  Certificate(
    this.id,
    this.date,
    this.icon,
  );

  final String id;
  final String date;
  final IconData icon;
}
