import 'dart:math';
import 'package:flutter_firebase_basic_tools/widgets/class_quiz_question.dart';
import 'package:flutter_firebase_basic_tools/widgets/class_quiz_result.dart';
import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/logic/get_user_info.dart';
import 'package:flutter_firebase_basic_tools/logic/email_sender.dart';
import 'package:flutter_firebase_basic_tools/logic/generate_certificate_html_from_values.dart';
import 'package:flutter_firebase_basic_tools/logic/timestamp_to_date.dart';
import 'package:flutter_firebase_basic_tools/logic/translate.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ClassQuiz extends StatefulWidget {
  const ClassQuiz({
    @required this.courseID,
    @required this.courseName,
    @required this.unitID,
    @required this.lastUnit,
    @required this.apiKey,
  });

  final String courseID;
  final String courseName;
  final String unitID;
  final bool lastUnit;
  final String apiKey;

  @override
  _ClassQuizState createState() => _ClassQuizState();
}

class _ClassQuizState extends State<ClassQuiz> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  SharedPreferences prefs;

  String uid = "";

  Map<String, dynamic> userInfo = {};

  String currentLanguage = "en";

  CustomColors customColors = CustomColors();

  TranslationStream translationStream = TranslationStream();

  List<String> textList = [
    "Lives:",
    "Validate",
    "Progress:",
  ];

  bool quizPassed = false;

  List questionsResult = List();

  double percentageProgress = 0;

  int lives = 3;

  final PageController pageController = PageController(initialPage: 0);

  int currentPage = 0;

  List<Widget> widgetsList = List<Widget>();

  String htmlCertificate = "";

  List<Widget> widgetsAction(bool isPortrait) {
    return [
      Container(
        width: isPortrait ? 100 : 150,
        child: LanguagePicker(
          currentLanguage: currentLanguage,
          parentWidget: this,
          itemsTextColor: CustomColors().oceanBlue,
          apiKey: "[API_KEY]",
        ),
      ),
    ];
  }

  getQuizData(String unitID) {
    FirebaseFirestore.instance
        .collection('templates')
        .doc("certificate")
        .get()
        .then((DocumentSnapshot ds) {
      htmlCertificate = ds.data()["html"];
    });

    FirebaseFirestore.instance
        .collection('quizzes')
        .doc(unitID)
        .get()
        .then((DocumentSnapshot ds) {
      List questionsObject = ds.data()["questions"];

      questionsObject.shuffle();

      for (var i = 0; i < questionsObject.length; i++) {
        questionsResult.add(false);

        List finalPossibleAnswers = [];

        List currentAnswers = questionsObject[i]["answers"];

        if (currentAnswers.length > 2) {
          while (finalPossibleAnswers.length <
              (currentAnswers.length == 3 ? 2 : 3)) {
            print(finalPossibleAnswers);

            final _random = new Random();

            String randomPossibleAnswer =
                currentAnswers[_random.nextInt(currentAnswers.length)]
                    .toString();

            if (randomPossibleAnswer !=
                questionsObject[i]["correct_answer"].toString()) {
              bool randomPossibleAnswerAlreadyExist = false;

              for (var fourPossibleAnswersElement in finalPossibleAnswers) {
                if (fourPossibleAnswersElement == randomPossibleAnswer) {
                  randomPossibleAnswerAlreadyExist = true;
                }
              }

              if (!randomPossibleAnswerAlreadyExist) {
                finalPossibleAnswers.add(randomPossibleAnswer);
              }
            }
          }

          finalPossibleAnswers
              .add(questionsObject[i]["correct_answer"].toString());
        } else {
          finalPossibleAnswers = currentAnswers;
        }

        widgetsList.add(ClassQuizQuestion(
          questionTitle: questionsObject[i]["question_title"],
          answers: finalPossibleAnswers,
          demos: questionsObject[i]["demos"],
          classQuiz: this,
          correctAnswer: questionsObject[i]["correct_answer"].toString(),
          questionID: i,
          apiKey: "[API_KEY]",
        ));
      }

      widgetsList.add(ClassQuizResult(
        classQuiz: this,
      ));

      setState(() {});

      translationStream.init(textList, this);
      translationStream.translate(widget.apiKey);
    });
  }

  getNextQuestion(bool answerIscorrect, int questionID) {
    if (answerIscorrect) {
      questionsResult[questionID] = true;
    } else {
      lives--;
    }

    if (lives == 0) {
      quizPassed = false;

      pageController.animateToPage(widgetsList.length - 1,
          duration: Duration(milliseconds: 800), curve: Curves.elasticOut);
    } else {
      List possibleNextPageIndex = List();

      for (int i = 0; i < questionsResult.length; i++) {
        if (!questionsResult[i]) {
          possibleNextPageIndex.add(i);
        }
      }

      percentageProgress =
          (100 * (questionsResult.length - possibleNextPageIndex.length)) /
              questionsResult.length;

      if (possibleNextPageIndex.length > 0) {
        int nextPageIndex = 0;

        if (possibleNextPageIndex.length > 1) {
          nextPageIndex = possibleNextPageIndex
              .firstWhere((possible) => possible != currentPage);
        } else {
          nextPageIndex = possibleNextPageIndex[0];
        }

        pageController.animateToPage(nextPageIndex,
            duration: Duration(milliseconds: 800), curve: Curves.elasticOut);
      } else {
        quizPassed = true;

        pageController.animateToPage(widgetsList.length - 1,
            duration: Duration(milliseconds: 800), curve: Curves.elasticOut);

        FirebaseFirestore.instance.collection("users").doc(uid).update({
          "units_completed": FieldValue.arrayUnion([widget.unitID]),
        }).catchError((err) => print(err));

        if (widget.lastUnit) {
          checkIfExistCertificate();
        }
      }

      setState(() {});
    }
  }

  generateCertificate(bool existCertificate) {
    if (!existCertificate) {
      FirebaseFirestore.instance
          .collection("certificates")
          .add({
            "course_id": widget.courseID,
            "date": FieldValue.serverTimestamp(),
            "user_id": uid,
          })
          .then((newDocument) => {
        FirebaseFirestore.instance
                    .collection("users")
                    .doc(uid)
                    .update({
                  "certificates":
                      FieldValue.arrayUnion([newDocument.id]),
                }).catchError((err) => print(err)),
        FirebaseFirestore.instance
                    .collection('templates')
                    .doc("certificate")
                    .get()
                    .then((DocumentSnapshot ds) async {
                  String htmlCertificate = ds.data()["html"];

                  DocumentSnapshot newDocumentSnapshot =
                      await newDocument.get();

                  String htmlCertificateResult =
                      await HtmlCertificateFromValues().generate(
                    courseName: widget.courseName,
                    userName:
                        userInfo["firstname"] + " " + userInfo["lastname"],
                    date:
                        TimestampToDate().convert(newDocumentSnapshot.data()["date"]),
                    id: newDocument.id,
                  );

                  User authUser = FirebaseAuth.instance.currentUser;

                  EmailSender()
                      .send(
                        to: authUser.email,
                        subject:
                            "${userInfo["firstname"] + " " + userInfo["lastname"]}, here is your certificate for Lean Six Sigma",
                        text: "New Message",
                        html: htmlCertificateResult,
                      )
                      .then((value) => {
                            _scaffoldKey.currentState.showSnackBar(SnackBar(
                              content: Text("Certificate sent! ✉️"),
                              duration: Duration(seconds: 3),
                            )),
                          })
                      .catchError((err) => print(err));
                }),
              })
          .catchError((err) => print(err));
    } else {
      _scaffoldKey.currentState.showSnackBar(SnackBar(
        content: Text("You already have this certificate 👍"),
        duration: Duration(seconds: 3),
      ));
    }
  }

  checkIfExistCertificate() async {
    bool userAlreadyHasCoursesCertificate = false;

    await FirebaseFirestore.instance
        .collection('users')
        .doc(uid)
        .get()
        .then((DocumentSnapshot ds1) {
      List certificates = ds1.data()["certificates"];

      if (certificates == null || certificates.length == 0) {
        generateCertificate(false);
      } else {
        print(certificates);

        for (int i = 0; i < certificates.length; i++) {
          FirebaseFirestore.instance
              .collection('certificates')
              .doc(certificates[i])
              .get()
              .then((DocumentSnapshot ds2) {
            print(ds2.data);

            print("course_id: " + ds2.data()["course_id"]);
            print("widget.courseID: " + widget.courseID);

            if (ds2.data()["course_id"] == widget.courseID) {
              userAlreadyHasCoursesCertificate = true;
            }

            if (i == certificates.length - 1) {
              print("final checkIfExistCertificate: " +
                  userAlreadyHasCoursesCertificate.toString());

              generateCertificate(userAlreadyHasCoursesCertificate);
            }
          });
        }
      }
    });
  }

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
    uid = prefs.getString("uid");
    userInfo = await getUserInfo(uid);
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
    getQuizData(widget.unitID);
  }

  @override
  Widget build(BuildContext context) {
    double appBarSize = 65;

    bool isPortrait =
        MediaQuery.of(context).orientation == Orientation.portrait;

    return Scaffold(
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: customColors.darkAqua.withOpacity(0.7),
          size: appBarSize,
          hasBackButton: true,
          actions: widgetsAction(isPortrait),
          customLeading: null,
        ),
      ),
      body: Container(
        child: widgetsList.length == 0
            ? Center(
                child: CircularProgressIndicator(
                  valueColor: new AlwaysStoppedAnimation<Color>(
                    CustomColors().lightAqua,
                  ),
                ),
              )
            : Column(
                mainAxisAlignment: MainAxisAlignment.spaceAround,
                children: <Widget>[
                  Container(
                    margin: EdgeInsets.only(
                      top: 15,
                      bottom: 15,
                    ),
                    child: Text(textList[0] + " " + lives.toString()),
                  ),
                  Container(
                    height: MediaQuery.of(context).size.height / 1.33,
                    child: PageView(
                      scrollDirection: Axis.horizontal,
                      physics: NeverScrollableScrollPhysics(),
                      onPageChanged: (int page) {
                        setState(() {
                          currentPage = page;
                        });
                      },
                      pageSnapping: true,
                      controller: pageController,
                      children: widgetsList,
                    ),
                  ),
                  Container(
                    margin: EdgeInsets.only(
                      top: 15,
                      bottom: 15,
                    ),
                    child:
                        Text(textList[2] + " " + percentageProgress.toString()),
                  ),
                ],
              ),
      ),
    );
  }
}
