import 'package:flutter_firebase_basic_tools/logic/url_launcher.dart';
import 'package:flutter_firebase_basic_tools/widgets/background_image_with_gradient_color.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:flutter_firebase_basic_tools/logic/my_flutter_app_icons.dart' as custicon;

class ContactUsContainer extends StatefulWidget {
  const ContactUsContainer({
    @required this.texts,
    @required this.landingClass,
  });

  final List<String> texts;
  final landingClass;

  @override
  _ContactUsContainerState createState() => _ContactUsContainerState();
}

class _ContactUsContainerState extends State<ContactUsContainer> {
  String facebookUrl = "https://www.facebook.com/abeinstitute/";
  String youtubeUrl =
      "https://www.youtube.com/channel/UCq09YKDjcoLpujRI7XUd2Zg";
  String instagramUrl =
      "https://instagram.com/abe_institute?igshid=hf7zdhgf3gft";
  String twitterUrl = "https://twitter.com/ABE_Institute";

  TextEditingController nameInputController = TextEditingController();
  TextEditingController emailInputController = TextEditingController();
  TextEditingController subjectInputController = TextEditingController();
  TextEditingController messageInputController = TextEditingController();

  @override
  Widget build(BuildContext context) {
    bool isPortrait =
        MediaQuery.of(context).orientation == Orientation.portrait;

    return Container(
      height: isPortrait
          ? (MediaQuery.of(context).size.height * 1.8)
          : (MediaQuery.of(context).size.height),
      width: MediaQuery.of(context).size.width,
      child: BackgroundImageWithGradientColor(
        height: isPortrait
            ? (MediaQuery.of(context).size.height * 1.8)
            : (MediaQuery.of(context).size.height),
        width: MediaQuery.of(context).size.width,
        boxFit: BoxFit.cover,
        backgroundImagePath: "assets/images/background_building.jpg",
        linearGradient: LinearGradient(
          begin: FractionalOffset.centerLeft,
          end: FractionalOffset.centerRight,
          colors: [
            Colors.black.withOpacity(0.7),
            Colors.black.withOpacity(0.7),
          ],
          stops: [0.0, 1.0],
        ),
        child: Column(
          children: <Widget>[
            Spacer(flex: 1),
            Expanded(
              flex: 1,
              child: FractionallySizedBox(
                widthFactor: 0.7,
                child: Text(
                  widget.texts[0],
                  textAlign: TextAlign.center,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 40,
                    //fontWeight: FontWeight.bold,
                  ),
                ),
              ),
            ),
            Expanded(
              flex: isPortrait ? 2 : 1,
              child: FractionallySizedBox(
                widthFactor: 0.55,
                child: Text(
                  widget.texts[1],
                  textAlign: TextAlign.center,
                  style: TextStyle(
                    color: Colors.white,
                    fontSize: 18,
                  ),
                ),
              ),
            ),
            Expanded(
              flex: isPortrait ? 10 : 6,
              child: FractionallySizedBox(
                widthFactor: isPortrait ? 0.8 : 0.8,
                child: Card(
                  elevation: 5,
                  child: FractionallySizedBox(
                    widthFactor: isPortrait ? 0.8 : 1,
                    child: Flex(
                      direction: isPortrait ? Axis.vertical : Axis.horizontal,
                      mainAxisAlignment: MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: <Widget>[
                        Spacer(flex: 1),
                        Expanded(
                          flex: 10,
                          child: Column(
                            children: <Widget>[
                              isPortrait ? Container() : Spacer(flex: 1),
                              Expanded(
                                flex: 1,
                                child: TextFormField(
                                  controller: nameInputController,
                                  decoration: InputDecoration(
                                    hintText: 'What do people call you?',
                                    labelText: widget.texts[2],
                                  ),
                                  onSaved: (String value) {},
                                  validator: (String value) {
                                    return value.contains('@')
                                        ? 'Do not use the @ char.'
                                        : null;
                                  },
                                ),
                              ),
                              Expanded(
                                flex: 1,
                                child: TextFormField(
                                  controller: emailInputController,
                                  decoration: InputDecoration(
                                    labelText: widget.texts[3],
                                  ),
                                  onSaved: (String value) {},
                                  validator: (String value) {
                                    return value.contains('@')
                                        ? 'Do not use the @ char.'
                                        : null;
                                  },
                                ),
                              ),
                              Expanded(
                                flex: 1,
                                child: TextFormField(
                                  controller: subjectInputController,
                                  decoration: InputDecoration(
                                    labelText: widget.texts[4],
                                  ),
                                  onSaved: (String value) {},
                                  validator: (String value) {
                                    return value.contains('@')
                                        ? 'Do not use the @ char.'
                                        : null;
                                  },
                                ),
                              ),
                              Expanded(
                                flex: 1,
                                child: TextFormField(
                                  controller: messageInputController,
                                  decoration: InputDecoration(
                                    labelText: widget.texts[5],
                                  ),
                                  onSaved: (String value) {},
                                  validator: (String value) {
                                    return value.contains('@')
                                        ? 'Do not use the @ char.'
                                        : null;
                                  },
                                ),
                              ),
                              Expanded(
                                flex: 1,
                                child: Row(
                                  children: <Widget>[
                                    isPortrait ? Spacer(flex: 1) : Container(),
                                    Expanded(
                                      flex: 2,
                                      child: RaisedButton(
                                        color: CustomColors().oceanBlue,
                                        onPressed: () {
                                          String newMessage = nameInputController
                                                  .text +
                                              " has a message for you! \n\n Email: " +
                                              emailInputController.text +
                                              "\n\n Message: " +
                                              messageInputController.text;

                                          FirebaseFirestore.instance
                                              .collection("emails")
                                              .doc()
                                              .set({
                                                "to":
                                                    'community-mgmt@abeinstitute.com',
                                                "message": {
                                                  "subject":
                                                      "Message from contact us section: " +
                                                          '"' +
                                                          subjectInputController
                                                              .text +
                                                          '"',
                                                  "text": newMessage,
                                                  //"html": 'This is the <code>HTML</code> section of the email body.',
                                                }
                                              })
                                              .then((value) => {
                                                    nameInputController.clear(),
                                                    emailInputController
                                                        .clear(),
                                                    subjectInputController
                                                        .clear(),
                                                    messageInputController
                                                        .clear(),
                                                    widget
                                                        .landingClass
                                                        .scaffoldKey
                                                        .currentState
                                                        .showSnackBar(SnackBar(
                                                      content: Text(
                                                          "✉️ Message sent! 👍"),
                                                      duration:
                                                          Duration(seconds: 2),
                                                    )),
                                                  })
                                              .catchError((err) => print(err));
                                        },
                                        child: Row(
                                          children: <Widget>[
                                            Spacer(flex: 1),
                                            Expanded(
                                              flex: 5,
                                              child: Icon(
                                                custicon.MyFlutterApp
                                                    .paper_plane_empty,
                                                color: Colors.white,
                                                size: 16,
                                              ),
                                            ),
                                            Spacer(flex: 1),
                                            Expanded(
                                              flex: 7,
                                              child: Text(
                                                widget.texts[6],
                                                textAlign: TextAlign.center,
                                                style: TextStyle(
                                                  color: Colors.white,
                                                  fontSize: 12,
                                                  //fontWeight: FontWeight.bold,
                                                ),
                                              ),
                                            ),
                                            Spacer(flex: 1),
                                          ],
                                        ),
                                      ),
                                    ),
                                    Spacer(flex: isPortrait ? 1 : 9),
                                  ],
                                ),
                              ),
                              isPortrait ? Container() : Spacer(flex: 1),
                            ],
                          ),
                        ),
                        isPortrait ? Container() : Spacer(flex: 1),
                        Expanded(
                          flex: isPortrait ? 10 : 4,
                          child: Column(
                            children: <Widget>[
                              Spacer(flex: isPortrait ? 1 : 4),
                              Expanded(
                                flex: 1,
                                child: Icon(
                                  Icons.location_on,
                                  color: CustomColors().oceanBlue,
                                ),
                              ),
                              Spacer(flex: 1),
                              Expanded(
                                flex: 3,
                                child: Text(
                                  widget.texts[7],
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    color: Colors.black,
                                    fontSize: 18,
                                    //fontWeight: FontWeight.bold,
                                  ),
                                ),
                              ),
                              Spacer(flex: 2),
                              Expanded(
                                flex: 1,
                                child: Icon(
                                  Icons.phone,
                                  color: CustomColors().oceanBlue,
                                ),
                              ),
                              Spacer(flex: 1),
                              Expanded(
                                flex: 2,
                                child: Text(
                                  widget.texts[8],
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    color: Colors.black,
                                    fontSize: 18,
                                    //fontWeight: FontWeight.bold,
                                  ),
                                ),
                              ),
                              Spacer(flex: 2),
                              Expanded(
                                flex: 1,
                                child: Icon(
                                  Icons.email,
                                  color: CustomColors().oceanBlue,
                                ),
                              ),
                              Spacer(flex: 1),
                              Expanded(
                                flex: 4,
                                child: Text(
                                  widget.texts[9],
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                    color: Colors.black,
                                    fontSize: 18,
                                    //fontWeight: FontWeight.bold,
                                  ),
                                ),
                              ),
                              Spacer(flex: 1),
                              Expanded(
                                flex: 3,
                                child: Row(
                                  children: <Widget>[
                                    Spacer(flex: 1),
                                    Expanded(
                                      flex: 5,
                                      child: IconButton(
                                        onPressed: () async {
                                          launchInBrowser(facebookUrl);
                                        },
                                        icon: Icon(
                                          custicon
                                              .MyFlutterApp.facebook_squared,
                                          color: CustomColors().oceanBlue,
                                        ),
                                      ),
                                    ),
                                    Expanded(
                                      flex: 5,
                                      child: IconButton(
                                        onPressed: () async {
                                          launchInBrowser(youtubeUrl);
                                        },
                                        icon: Icon(
                                          custicon.MyFlutterApp.youtube_play,
                                          color: CustomColors().oceanBlue,
                                        ),
                                      ),
                                    ),
                                    Expanded(
                                      flex: 5,
                                      child: IconButton(
                                        onPressed: () async {
                                          launchInBrowser(instagramUrl);
                                        },
                                        icon: Icon(
                                          custicon.MyFlutterApp.instagram,
                                          color: CustomColors().oceanBlue,
                                        ),
                                      ),
                                    ),
                                    Expanded(
                                      flex: 5,
                                      child: IconButton(
                                        onPressed: () async {
                                          launchInBrowser(twitterUrl);
                                        },
                                        icon: Icon(
                                          custicon.MyFlutterApp.twitter,
                                          color: CustomColors().oceanBlue,
                                        ),
                                      ),
                                    ),
                                    Spacer(flex: 1),
                                  ],
                                ),
                              ),
                              isPortrait ? Container() : Spacer(flex: 6),
                            ],
                          ),
                        ),
                        Spacer(flex: 1),
                      ],
                    ),
                  ),
                ),
              ),
            ),
            Spacer(flex: 1),
          ],
        ),
      ),
    );
  }
}
