import 'package:flutter_firebase_basic_tools/widgets/class_session.dart';
import 'package:flutter_firebase_basic_tools/widgets/language_picker.dart';
import 'package:flutter_firebase_basic_tools/widgets/topbar.dart';
import 'package:flutter_firebase_basic_tools/logic/get_user_info.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class CoursesList extends StatefulWidget {
  @override
  _CoursesListState createState() => _CoursesListState();
}

class _CoursesListState extends State<CoursesList> {
  SharedPreferences prefs;

  List<String> coursesBuyed = List<String>();
  List<Map<String, dynamic>> coursesAndUnits = List<Map<String, dynamic>>();
  List<Course> courses = List<Course>();

  Map<String, dynamic> userInfo = {};
  String uid;

  initSharedPreferences() async {
    prefs = await SharedPreferences.getInstance();
    uid = prefs.getString("uid");
    getCoursesAndUnits();
  }

  getCoursesAndUnits() async {
    userInfo = await getUserInfo(uid);

    courses.clear();

    if (userInfo["courses_buyed"] != null) {
      if (userInfo["courses_buyed"].length > 0) {
        coursesBuyed = List.from(userInfo["courses_buyed"]);

        for (int i = 0; i < coursesBuyed.length; i++) {
          DocumentSnapshot firestoreCourse = await FirebaseFirestore.instance
              .collection("courses")
              .doc(coursesBuyed[i])
              .get();

          List<String> units = List.from(firestoreCourse.data()["units"]);

          List<String> unitsName = List<String>.generate(
              units.length, (counter) => "Unit ${counter + 1}");

          List<bool> unitsCompletedStatus =
              List<bool>.generate(units.length, (counter) => false);

          for (int i = 0; i < units.length; i++) {
            if (userInfo["units_completed"] != null) {
              if (userInfo["units_completed"].length > 0) {
                for (int j = 0; j < userInfo["units_completed"].length; j++) {
                  if (units[i] == userInfo["units_completed"][j]) {
                    unitsCompletedStatus[i] = true;
                  }
                }
              }
            }
          }

          courses.add(Course(
            coursesBuyed[i],
            firestoreCourse.data()["name"],
            unitsName,
            Icons.check_circle_outline,
            units,
            unitsCompletedStatus,
          ));
        }
      }
    }

    setState(() {});
  }

  List<String> textList = [
    "Account",
    "Notifications",
    "My courses",
    "Buy courses",
    "Certificates and Rewards",
    "Settings",
  ];

  String currentLanguage = "en";

  CustomColors customColors = CustomColors();

  List<Widget> widgetsAction(bool isPortrait) {
    return [
      Container(
        width: isPortrait ? 100 : 150,
        child: LanguagePicker(
          currentLanguage: currentLanguage,
          parentWidget: this,
          itemsTextColor: CustomColors().oceanBlue,
          apiKey: "[API_KEY]",
        ),
      ),
    ];
  }

  @override
  void initState() {
    super.initState();
    initSharedPreferences();
  }

  @override
  Widget build(BuildContext context) {
    double appBarSize = 65;

    bool isPortrait =
        MediaQuery.of(context).orientation == Orientation.portrait;

    return Scaffold(
      extendBodyBehindAppBar: true,
      appBar: PreferredSize(
        preferredSize: Size.fromHeight(appBarSize),
        child: Topbar(
          backgroundColor: customColors.darkAqua.withOpacity(0.7),
          size: appBarSize,
          hasBackButton: true,
          actions: widgetsAction(isPortrait),
          customLeading: null,
        ),
      ),
      body: courses.length > 0
          ? Align(
              alignment: Alignment.bottomCenter,
              child: FractionallySizedBox(
                heightFactor: 0.8,
                widthFactor: 0.8,
                child: ListView.builder(
                  itemCount: courses.length,
                  itemBuilder: (context, i) {
                    return Column(
                      children: <Widget>[
                        ExpansionTile(
                          backgroundColor: Colors.grey[100],
                          title: Text(
                            courses[i].title,
                            style: TextStyle(
                              fontSize: 20.0,
                              fontWeight: FontWeight.bold,
                              fontStyle: FontStyle.italic,
                            ),
                          ),
                          children: <Widget>[
                            Column(
                              children: _buildExpandableContent(
                                courses[i],
                                context,
                                getCoursesAndUnits,
                              ),
                            ),
                          ],
                        ),
                      ],
                    );
                  },
                ),
              ),
            )
          : Center(
              child: CircularProgressIndicator(
                valueColor: new AlwaysStoppedAnimation<Color>(
                  CustomColors().lightAqua,
                ),
              ),
            ),
    );
  }
}

_buildExpandableContent(
    Course course, BuildContext context, Function getCoursesAndUnits) {
  List<Widget> columnContent = [];

  for (String content in course.contents) {
    int contentIndex = course.contents.indexOf(content);
    columnContent.add(
      GestureDetector(
        onTap: () async {
          String unitID = course.unitIDs[contentIndex];

          /*Navigator.push(
            context,
            MaterialPageRoute(
                builder: (context) => ClassSession(
                      courseID: course.id,
                      courseName: course.title,
                      unitID: unitID,
                    )),
          );*/

          if (contentIndex == 0) {
            await Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (context) => ClassSession(
                        courseID: course.id,
                        courseName: course.title,
                        unitID: unitID,
                        apiKey: "[API_KEY]",
                      )),
            );
            getCoursesAndUnits();
          } else {
            if (course.unitsCompletedStatus[contentIndex - 1]) {
              Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (context) => ClassSession(
                          courseID: course.id,
                          courseName: course.title,
                          unitID: unitID,
                          apiKey: "[API_KEY]",
                        )),
              );
            }
          }
        },
        child: ListTile(
          title: Text(
            content,
            style: TextStyle(fontSize: 18.0),
          ),
          leading: Icon(
            course.icon,
            color: course.unitsCompletedStatus[contentIndex]
                ? Colors.lightGreen
                : Colors.transparent,
          ),
        ),
      ),
    );
  }

  return columnContent;
}

class Course {
  Course(
    this.id,
    this.title,
    this.contents,
    this.icon,
    this.unitIDs,
    this.unitsCompletedStatus,
  );

  final String id;
  final String title;
  List<String> contents = [];
  final IconData icon;
  List<String> unitIDs = [];
  List<bool> unitsCompletedStatus = [];
}
