import 'package:flutter_firebase_basic_tools/widgets/background_image_with_gradient_color.dart';
import 'package:flutter_firebase_basic_tools/logic/custom_colors.dart';
import 'package:auto_size_text/auto_size_text.dart';
import 'package:flutter/material.dart';
import 'package:flutter/painting.dart';

class IntroductionContainer extends StatefulWidget {
  const IntroductionContainer({
    @required this.texts,
  });

  final List<String> texts;

  @override
  IntroductionContainerState createState() => IntroductionContainerState();
}

class IntroductionContainerState extends State<IntroductionContainer>
    with SingleTickerProviderStateMixin {
  AnimationController animationController;
  Animation<double> animation;

  @override
  void initState() {
    super.initState();

    animationController =
        AnimationController(vsync: this, duration: Duration(milliseconds: 700));

    Animation curve =
        CurvedAnimation(parent: animationController, curve: Curves.easeInQuad);

    animation = Tween<double>(begin: 0, end: 30).animate(curve)
      ..addListener(() {
        if (animationController.isCompleted) {
          animationController.reverse();
        } else if (animationController.value == 0) {
          animationController.forward();
        }
        setState(() {});
      });

    animationController.forward();
  }

  @override
  void dispose() {
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    bool isPortrait =
        MediaQuery.of(context).orientation == Orientation.portrait;

    return BackgroundImageWithGradientColor(
      height: MediaQuery.of(context).size.height,
      width: MediaQuery.of(context).size.width,
      boxFit: BoxFit.cover,
      backgroundImagePath: "assets/images/introduction_container.jpg",
      linearGradient: LinearGradient(
        begin: FractionalOffset.centerLeft,
        end: FractionalOffset.centerRight,
        colors: [
          Colors.lightBlueAccent.withOpacity(0.0),
          CustomColors().oceanBlue.withOpacity(0.0),
        ],
        stops: [0.0, 1.0],
      ),
      child: Center(
        child: Stack(
          children: <Widget>[
            Container(
              child: Center(
                child: Column(
                  children: <Widget>[
                    Spacer(flex: 12),
                    Expanded(
                      flex: 16,
                      child: Container(
                        width: isPortrait ? 250 : 400,
                        decoration: BoxDecoration(
                          shape: BoxShape.rectangle,
                          borderRadius: BorderRadius.all(Radius.circular(20)),
                          image: DecorationImage(
                            fit: BoxFit.fitWidth,
                            image: AssetImage(
                              "assets/images/abei_logo.png",
                            ),
                          ),
                        ),
                      ),
                    ),
                    Expanded(
                      flex: isPortrait ? 12 : 5,
                      child: FractionallySizedBox(
                        widthFactor: isPortrait ? 0.9 : 1,
                        child: Column(
                          children: [
                            Expanded(
                              flex: isPortrait ? 4 : 10,
                              child: AutoSizeText(
                                widget.texts[0],
                                textAlign: TextAlign.center,
                                style: TextStyle(
                                  color: Colors.white,
                                  fontSize: 36,
                                  fontWeight: FontWeight.bold,
                                ),
                                minFontSize: 20,
                                maxLines: 2,
                                overflow: TextOverflow.clip,
                              ),
                            ),
                            Spacer(flex: 1),
                            Expanded(
                              flex: isPortrait ? 4 : 10,
                              child: SelectableText(
                                widget.texts[1],
                                toolbarOptions: ToolbarOptions(
                                  copy: true,
                                  paste: true,
                                ),
                                textAlign: TextAlign.center,
                                style: TextStyle(
                                  color: Colors.white,
                                  fontSize: 16,
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ),
                    Expanded(
                      flex: 2,
                      child: Transform.translate(
                        offset: Offset(0, animation.value),
                        child: Icon(
                          Icons.keyboard_arrow_down,
                          color: Colors.orangeAccent,
                          size: 40,
                        ),
                      ),
                    ),
                    Spacer(flex: 2),
                  ],
                ),
              ),
            ),
            /*Container(
              color: Colors.transparent,
              height: MediaQuery.of(context).size.height,
              width: MediaQuery.of(context).size.width,
              child: CustomPaint(
                painter: CurvePainter(),
              ),
            ),*/
          ],
        ),
      ),
    );
  }
}

class CurvePainter extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    CustomColors customColors = CustomColors();

    var rect = Offset.zero & size;

    var paint = Paint();

    paint.shader = LinearGradient(
      begin: Alignment.topCenter,
      end: Alignment.bottomCenter,
      colors: [
        customColors.oceanBlue,
        customColors.green,
      ],
    ).createShader(rect);

    paint.style = PaintingStyle.fill; // Change this to fill

    var path = Path();

    path.moveTo(size.width * 0.02, 0);
    path.quadraticBezierTo(size.width * 0.04, size.height * 0.25,
        size.width * 0.02, size.height * 0.5);
    path.quadraticBezierTo(size.width * 0.0, size.height * 0.75,
        size.width * 0.02, size.height * 1.0);
    path.lineTo(0, size.height);
    path.lineTo(0, 0);

    canvas.drawPath(path, paint);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    return true;
  }
}
