import 'package:flutter_firebase_basic_tools/widgets/background_image_with_gradient_color.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

enum UserInfoFormType {
  login,
  register,
  forgotPassword,
  accountInfo,
  custom,
}

class UserInfoFormContainer extends StatefulWidget {
  const UserInfoFormContainer({
    this.userInfoFormType,
    this.backgroundImagePath,
    this.linearGradient,
    this.child,
    this.cardColor,
  });

  final UserInfoFormType userInfoFormType;
  final String backgroundImagePath;
  final LinearGradient linearGradient;
  final Widget child;
  final Color cardColor;

  @override
  _UserInfoFormContainerState createState() => _UserInfoFormContainerState();
}

class _UserInfoFormContainerState extends State<UserInfoFormContainer> {
  @override
  void initState() {
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    bool isLandscape =
        (MediaQuery.of(context).orientation == Orientation.landscape);

    double height;
    double width;

    switch (widget.userInfoFormType) {
      case UserInfoFormType.login:
        {
          height = 450;
          width = isLandscape ? 430 : 330;
        }
        break;

      case UserInfoFormType.register:
        {
          height = 600;
          width = isLandscape ? 650 : 330;
        }
        break;

      case UserInfoFormType.forgotPassword:
        {
          height = 350;
          width = isLandscape ? 430 : 330;
        }
        break;

      case UserInfoFormType.accountInfo:
        {
          height = 600;
          width = isLandscape ? 650 : 330;
        }
        break;

      case UserInfoFormType.custom:
        {
          height = 450;
          width = isLandscape ? 430 : 330;
        }
        break;
    }

    return BackgroundImageWithGradientColor(
      height: MediaQuery.of(context).size.height,
      width: MediaQuery.of(context).size.width,
      boxFit: BoxFit.cover,
      backgroundImagePath: widget.backgroundImagePath,
      linearGradient: widget.linearGradient,
      blurImage: true,
      blurImageColor: Colors.black.withOpacity(0.1),
      blurImageParameters: [0.7, 0.7],
      //child: Center(
      child: Container(
        height: height,
        width: width,
        child: Card(
          elevation: 5,
          color: widget.cardColor,
          child: FractionallySizedBox(
            heightFactor: 0.95,
            child: Container(
              padding: EdgeInsets.only(
                left: 30,
                right: isLandscape
                    ? 30
                    : widget.userInfoFormType == UserInfoFormType.accountInfo
                        ? 10
                        : 30,
              ),
              child: widget.child,
            ),
          ),
        ),
      ),
    );
  }
}
